@@ -224,6 +224,11 @@ check_range (GtkSpellChecker *spell, GtkTextIter start,
       wend = wstart;
       gtk_text_iter_forward_word_end (&wend);
 
+      /* make sure we've actually advanced
+       * (we don't advance in some corner cases, such as after punctuation) */
+      if (gtk_text_iter_equal (&wstart, &wend))
+        break;
+
       inword = (gtk_text_iter_compare (&wstart, &cursor) < 0) &&
                (gtk_text_iter_compare (&cursor, &wend) <= 0);
 
@@ -1187,6 +1192,24 @@ gtk_spell_checker_set_language (GtkSpellChecker *spell, const gchar *lang, GErro
   return ret;
 }
 
+/**
+ * gtk_spell_checker_check_word:
+ * @spell: The #GtkSpellChecker object.
+ * @word: The word to check.
+ *
+ * Check the specified word.
+ *
+ * Returns: TRUE if the word is correctly spelled, FALSE otherwise.
+ */
+gboolean
+gtk_spell_checker_check_word (GtkSpellChecker *spell, const gchar *word)
+{
+  if (g_unichar_isdigit (*word) == TRUE || /* don't check numbers */
+      enchant_dict_check (spell->priv->speller, word, strlen (word)) == 0)
+    return TRUE;
+  return FALSE;
+}
+
 /**
  * gtk_spell_checker_recheck_all:
  * @spell: The #GtkSpellChecker object.
@@ -1207,6 +1230,31 @@ gtk_spell_checker_recheck_all (GtkSpellChecker *spell)
     }
 }
 
+/**
+ * gtk_spell_checker_get_suggestions:
+ * @spell: A #GtkSpellChecker.
+ * @word: The word for which to fetch suggestions
+ *
+ * Retreives a list of spelling suggestions for the specified word.
+ *
+ * Returns: (transfer full) (element-type utf8): the list of spelling
+ * suggestions for the specified word, or NULL if there are no suggestions.
+ */
+GList*
+gtk_spell_checker_get_suggestions (GtkSpellChecker *spell, const gchar* word)
+{
+  char **suggestions;
+  size_t n_suggs, i;
+  GList* result = NULL;
+  suggestions = enchant_dict_suggest (spell->priv->speller, word,
+                                      strlen (word), &n_suggs);
+  for (i = 0; i < n_suggs; ++i)
+    {
+      result = g_list_append (result, g_strdup (suggestions[i]));
+    }
+  return result;
+}
+
 /**
  * gtk_spell_checker_get_suggestions_menu:
  * @spell: A #GtkSpellChecker.
